/** Put here CSS that should apply to editor screens only. **/

body {
    padding-top: 60px;
}

.input, input, textarea, select {
    width: 400px;
}

form.login label {
    width: 200px;
}

form.login div.input {
    margin-left: 220px;
}

/* deform workarounds while we wait for deform_bootstrap to become perfect */
form.deform ul {
    list-style: none;
    margin-left: 0;
}

form.deform div.input label {
    padding-top: 0;
    margin-right: 5px;
}

